@@ -10,6 +10,7 @@ from __future__ import annotations
 
 import abc
 import asyncio
+import enum
 import inspect
 import logging
 import pathlib
@@ -24,24 +25,42 @@ import yaml
 from sdsstools import get_logger, read_yaml_file
 from sdsstools.logger import SDSSLogger
 
-from clu.command import BaseCommand, Command
-
 from .model import Model
 from .store import KeywordStore
 from .tools import REPLY
 
 
 if TYPE_CHECKING:
+    from clu.command import BaseCommand, Command
     from clu.legacy.types.messages import Keywords
 
 
-__all__ = ["BaseClient", "BaseActor", "Reply"]
+__all__ = ["BaseClient", "BaseActor", "Reply", "MessageCode"]
 
 
 SchemaType = Union[Dict[str, Any], pathlib.Path, str]
 T = TypeVar("T", bound="BaseClient")
 
 
+class MessageCode(enum.Enum):
+    """Flags for message codes."""
+
+    STARTED = ">"
+    DONE = ":"
+    FAILED = "f"
+    ERROR = "e"
+    CRITICAL = "!"
+    INFO = "i"
+    WARNING = "w"
+    DEBUG = "d"
+
+    def __eq__(self, __o: object) -> bool:
+        if isinstance(__o, MessageCode):
+            return super().__eq__(__o)
+
+        return self.value == __o
+
+
 class BaseClient(metaclass=abc.ABCMeta):
     """A base client that can be used for listening or for an actor.
 
@@ -80,6 +99,8 @@ class BaseClient(metaclass=abc.ABCMeta):
 
     name: str
 
+    command_models: dict[str, dict] = {}
+
     def __init__(
         self,
         name: str,
@@ -91,9 +112,6 @@ class BaseClient(metaclass=abc.ABCMeta):
         validate: bool = True,
         config: dict = {},
     ):
-
-        self.loop = loop or asyncio.get_event_loop()
-
         self.name = name
         assert self.name, "name cannot be empty."
 
@@ -126,22 +144,19 @@ class BaseClient(metaclass=abc.ABCMeta):
         self.log.info("cancelling all pending tasks and shutting down.")
 
         tasks = [
-            task
-            for task in asyncio.all_tasks(loop=self.loop)
-            if task is not asyncio.current_task(loop=self.loop)
+            task for task in asyncio.all_tasks() if task is not asyncio.current_task()
         ]
         list(map(lambda task: task.cancel(), tasks))
 
         await asyncio.gather(*tasks, return_exceptions=True)
 
-        self.loop.stop()
+        asyncio.get_running_loop().stop()
 
     @staticmethod
     def _parse_config(
         input: Union[Dict[str, Any], pathlib.Path, str],
         loader=yaml.FullLoader,
     ) -> Dict[str, Any]:
-
         if not isinstance(input, dict):
             input = pathlib.Path(input)
             assert input.exists(), "configuration path does not exist."
@@ -217,13 +232,10 @@ class BaseClient(metaclass=abc.ABCMeta):
             log = get_logger("clu:" + self.name)
         else:
             assert isinstance(log, SDSSLogger), "Logger must be sdsstools.SDSSLogger"
-            self.log = log
-            return log
 
         log.setLevel(REPLY)
 
-        if log is not False and log_dir:
-
+        if log_dir:
             log_dir = pathlib.Path(log_dir).expanduser()
 
             log.start_file_logger(
@@ -233,7 +245,8 @@ class BaseClient(metaclass=abc.ABCMeta):
             )
 
             if log.fh:  # In case starting the file logger fails.
-                log.fh.formatter.converter = time.gmtime
+                if log.fh.formatter:
+                    log.fh.formatter.converter = time.gmtime
                 log.fh.setLevel(REPLY)
 
         log.sh.setLevel(logging.WARNING)
@@ -245,12 +258,17 @@ class BaseClient(metaclass=abc.ABCMeta):
         self.log = log
         self.log.debug(f"{self.name}: logging system initiated.")
 
-        # Set the loop exception handler to be handled by the logger.
-        self.loop.set_exception_handler(self.log.asyncio_exception_handler)
-
         return log
 
-    def send_command(self, actor: str, *args, **kwargs):  # pragma: no cover
+    def set_loop_exception_handler(self):
+        """Sets the lopp exception handler to be handled by the logger."""
+
+        if self.log:
+            # Set the loop exception handler to be handled by the logger.
+            loop = asyncio.get_running_loop()
+            loop.set_exception_handler(self.log.asyncio_exception_handler)
+
+    async def send_command(self, actor: str, *args, **kwargs) -> Command:
         """Sends a command to an actor and returns a `.Command` instance."""
 
         raise NotImplementedError(
@@ -288,7 +306,6 @@ class ProxyClient:
     """
 
     def __init__(self, client: BaseClient, actor: str):
-
         self.client = client
         self.actor = actor
 
@@ -348,7 +365,6 @@ class BaseActor(BaseClient):
         additional_properties: bool = False,
         **kwargs,
     ):
-
         super().__init__(*args, **kwargs)
 
         self.load_schema(schema, additional_properties=additional_properties)
@@ -416,24 +432,38 @@ class BaseActor(BaseClient):
         pass
 
     @abc.abstractmethod
-    def _write_internal(self, reply: Reply):
+    def _write_internal(self, reply: Reply, write_to_log: bool = True):
         """Internally handle the reply and output it to the users.
 
         Must handle converting the general `.Reply` to the specific format of the
         actor transport. Must also handle logging the reply.
+
+        Parameters
+        ----------
+        reply
+            The reply object to output to users.
+        write_to_log
+            Whether to write the reply to the log. Defaults to yes but
+            it may be useful to prevent large repetitive replies cluttering
+            the log.
+
         """
+
         pass
 
     def write(
         self,
-        message_code: str = "i",
+        message_code: MessageCode | str = MessageCode.INFO,
         message: Optional[Dict[str, Any] | str] = None,
         command: Optional[BaseCommand] = None,
         broadcast: bool = False,
         validate: bool | None = None,
         expand_exceptions: bool = True,
+        traceback_frame: int = 0,
         silent: bool = False,
-        call_internal: bool = True,
+        internal: bool = False,
+        emit: bool = True,
+        write_to_log: bool = True,
         **kwargs,
     ) -> Reply:
         """Writes a message to user(s).
@@ -481,22 +511,36 @@ class BaseActor(BaseClient):
             the actor global behaviour.
         expand_exceptions
             If the value of one of the keywords is an exception object and
-            `expand_exception=True`, the exception value will be converted into
-            a dictionary of ``exception_type`` and ``exception_message``. Otherwise
-            the exception will just be stringified.
+            `expand_exception=True`, the exception information and traceback will
+            be expanded. Otherwise the exception will just be stringified.
+        traceback_frame
+            The frame from the traceback to use if ``expand_exception=True``. By
+            default reports the file and line of the last traceback frame.
         silent
             When `True` does not output the message to the users. This can be used to
             issue internal commands that update the internal model but that don't
             clutter the output.
-        call_internal
+        internal
+            Marks the `.Reply` instance as internal. Internal replies are expected to
+            be emitted to the users, but with a header indicating that the message
+            is not for broader consumption. This can be useful when one wants to
+            reply with verbose information that can be skipped from CLI or logs.
+        emit
             Whether to call the actor internal write method. Should be `True` but
-            it's sometimes useful to call `.write` with ``call_internal=False`` when
-            one is overriding the method and wants to control when to call the internal
-            method.
+            it's sometimes useful to call `.write` with ``emit=False`` when
+            one is overriding the method and wants to control when to call the
+            internal method. In that case, a `.Reply` object is returned but nothing
+            is output to the users.
+        write_to_log
+            Whether to write the reply to the log. Defaults to yes but
+            it may be useful to prevent large repetitive replies cluttering
+            the log.
         kwargs
             Keyword arguments that will used to update the message.
         """
 
+        message_code = MessageCode(message_code)
+
         if isinstance(message, str):
             message = {"text": message}
         else:
@@ -512,15 +556,36 @@ class BaseActor(BaseClient):
         for key, value in message.items():
             if isinstance(value, Exception):
                 if expand_exceptions is True:
+                    filename: str | None = None
+                    lineno: int | None = None
+                    if value.__traceback__ is not None:
+                        tb = value.__traceback__
+                        for _ in range(traceback_frame):
+                            t_next = tb.tb_next
+                            if t_next is None:
+                                break
+                            tb = t_next
+
+                        filename = tb.tb_frame.f_code.co_filename if tb else None
+                        lineno = tb.tb_lineno if tb else None
+
                     message[key] = {
-                        "exception_module": value.__class__.__module__,
-                        "exception_type": value.__class__.__name__,
-                        "exception_message": str(value),
+                        "module": value.__class__.__module__,
+                        "type": value.__class__.__name__,
+                        "message": str(value),
+                        "filename": filename,
+                        "lineno": lineno,
                     }
                 else:
                     message[key] = str(value)
 
-        reply = Reply(message_code, message, command=command, broadcast=broadcast)
+        reply = Reply(
+            message_code,
+            message,
+            command=command,
+            broadcast=broadcast,
+            internal=internal,
+        )
 
         do_validate = validate if validate is not None else self.validate
         if do_validate and self.model is not None:
@@ -535,7 +600,7 @@ class BaseActor(BaseClient):
                 else:
                     message = {"error": f"Failed validating the reply: {err}"}
 
-                reply.message_code = "e"
+                reply.message_code = MessageCode.ERROR
                 reply.message = message
                 reply.validated = False
             else:
@@ -544,11 +609,16 @@ class BaseActor(BaseClient):
         if command:
             command.replies.append(reply)
 
-        if call_internal and silent is False:
+        if emit and silent is False:
             if asyncio.iscoroutinefunction(self._write_internal):
-                asyncio.create_task(self._write_internal(reply))  # type: ignore
+                asyncio.create_task(
+                    self._write_internal(
+                        reply,
+                        write_to_log=write_to_log,
+                    )
+                )
             else:
-                self._write_internal(reply)
+                self._write_internal(reply, write_to_log=write_to_log)
 
         if self.store is not None:
             self.store.add_reply(reply)
@@ -568,22 +638,24 @@ class Reply:
 
     def __init__(
         self,
-        message_code: str,
+        message_code: MessageCode | str,
         message: Dict[str, Any],
         command: Optional[BaseCommand] = None,
         broadcast: bool = False,
         use_validation: bool = False,
         validated: bool = False,
+        internal: bool = False,
         keywords: Optional[Keywords] = None,
     ):
         self.date = datetime.utcnow()
-        self.message_code = message_code
+        self.message_code = MessageCode(message_code)
         self.message = message
         self.command = command
         self.broadcast = broadcast
         self.use_validation = use_validation
         self.validated = validated
         self.keywords = keywords
+        self.internal = internal
 
     @property
     def body(self):
