@@ -13,13 +13,17 @@ import json
 import logging
 import pathlib
 import uuid
+import warnings
+from copy import deepcopy
 
-from typing import TYPE_CHECKING, Awaitable, Callable, Dict, List, Optional, Union
+from typing import TYPE_CHECKING, Any, Awaitable, Callable, Dict, List, Optional, Union
 
 import aio_pika as apika
 
 from sdsstools.logger import SDSSLogger
 
+from clu.exceptions import CluWarning
+
 from .base import BaseClient, Reply
 from .command import Command
 from .model import ModelSet
@@ -220,11 +224,13 @@ class AMQPClient(BaseClient):
             assert isinstance(self.connection.connection, apika.Connection)
             url = str(self.connection.connection.url)
 
-        return f"<{str(self)} (name={self.name!r}, {url}>"
+        return f"<{str(self)} (name={self.name!r}, {url})>"
 
     async def start(self, exchange_name: str = __EXCHANGE_NAME__):
         """Starts the connection to the AMQP broker."""
 
+        self.set_loop_exception_handler()
+
         # Starts the connection and creates the exchange
         await self.connection.connect(exchange_name)
 
@@ -254,6 +260,11 @@ class AMQPClient(BaseClient):
         if self.connection.connection and not self.connection.connection.is_closed:
             await self.connection.stop()
 
+    def is_connected(self):
+        """Is the client connected to the exchange?"""
+
+        return self.connection.connection and not self.connection.connection.is_closed
+
     async def run_forever(self):
         """Runs the event loop forever."""
 
@@ -262,6 +273,20 @@ class AMQPClient(BaseClient):
         while not self.connection.connection.is_closed:
             await asyncio.sleep(1)
 
+    async def __aenter__(self):
+        """Starts the client inside a context manager."""
+
+        try:
+            if not self.is_connected():
+                await self.start()
+        except Exception:
+            await self.stop()
+
+    async def __aexit__(self, *_):
+        """Exits the context manager."""
+
+        await self.stop()
+
     async def handle_reply(
         self,
         message: apika.abc.AbstractIncomingMessage,
@@ -322,7 +347,7 @@ class AMQPClient(BaseClient):
             if asyncio.iscoroutinefunction(cb):
                 asyncio.create_task(cb(reply))
             else:
-                asyncio.get_running_loop().call_soon(cb, reply)
+                cb(reply)
 
         return reply
 
@@ -373,8 +398,6 @@ class AMQPClient(BaseClient):
 
         """
 
-        assert self.replies_queue
-
         if command and command.command_id:
             command_id = str(command.command_id)
         else:
@@ -398,32 +421,60 @@ class AMQPClient(BaseClient):
             consumer_id=consumer,
             internal=internal,
             actor=None,
-            loop=self.loop,
             reply_callback=callback,
             time_limit=time_limit,
         )
 
         self.running_commands[command_id] = command
 
-        headers: HeadersType = {
+        headers = {
             "command_id": command_id,
             "commander_id": commander_id,
             "internal": internal,
         }
 
-        # The routing key has the topic command and the name of
-        # the commanded actor.
-        routing_key = f"command.{consumer}"
-
         message_body = {"command_string": command_string}
 
+        await self._publish_message(
+            consumer,
+            headers=headers,
+            body=message_body,
+            correlation_id=command_id,
+        )
+
+        if await_command:
+            await command
+
+        return command
+
+    async def _publish_message(
+        self,
+        consumer: str,
+        headers: HeadersType = {},
+        body: dict[str, Any] = {},
+        correlation_id: str | None = None,
+    ):
+        """Publishes a message to an exchange."""
+
+        if not hasattr(self.connection, "exchange"):
+            warnings.warn(
+                f"Exchange is not ready to output message: {body}",
+                CluWarning,
+            )
+            return
+
+        assert self.replies_queue
+
+        # The routing key has the topic command and the name of the commanded actor.
+        routing_key = f"command.{consumer}"
+
         try:
             await self.connection.exchange.publish(
                 apika.Message(
-                    json.dumps(message_body).encode(),
+                    json.dumps(body).encode(),
                     content_type="text/json",
                     headers=headers,
-                    correlation_id=command_id,
+                    correlation_id=correlation_id,
                     reply_to=self.replies_queue.name,
                 ),
                 routing_key=routing_key,
@@ -439,15 +490,51 @@ class AMQPClient(BaseClient):
                     json.dumps(error_msg).encode(),
                     content_type="text/json",
                     headers=headers,
-                    correlation_id=command_id,
+                    correlation_id=correlation_id,
                 ),
                 routing_key=f"reply.{self.name}",
             )
 
-        if await_command:
-            await command
+    async def send_task(
+        self,
+        consumer: str,
+        task_name: str,
+        payload: dict[str, Any] = {},
+        **kwargs,
+    ):
+        """
 
-        return command
+        Parameters
+        ----------
+        consumer
+            The actor we are commanding.
+        task_name
+            The task to execute in the remote actor
+        payload
+            A serialisable dictionary with the payload to pass to the task.
+        kwargs
+            Additional arguments used to update the payload dictionary.
+
+        """
+
+        assert self.replies_queue
+
+        payload = deepcopy(payload)
+        payload.update(kwargs)
+
+        headers = {"commander_id": f"{self.name}.{consumer}", "task": True}
+
+        message_body = {"task": task_name}
+        message_body.update(payload)
+
+        correlation_id = str(uuid.uuid4())
+
+        await self._publish_message(
+            consumer,
+            headers=headers,
+            body=message_body,
+            correlation_id=correlation_id,
+        )
 
     def add_reply_callback(self, callback_func: ReplyCallbackType):
         """Adds a callback that is called when a new reply is received."""
