@@ -145,7 +145,6 @@ class BaseLegacyActor(BaseActor):
         self._server = TCPStreamServer(
             host,
             port,
-            loop=self.loop,
             connection_callback=self.new_user,
             data_received_callback=self.new_command,
         )
@@ -187,6 +186,8 @@ class BaseLegacyActor(BaseActor):
 
         """
 
+        self.set_loop_exception_handler()
+
         await self._server.start()
         self.log.info(f"running TCP server on {self.host}:{self.port}")
 
@@ -240,7 +241,7 @@ class BaseLegacyActor(BaseActor):
                 self.log.debug(f"user {transport.user_id} disconnected.")
                 return self.transports.pop(transport.user_id)
 
-        curr_ids = set(self.transports.keys())
+        curr_ids: set[int] = set(self.transports.keys())
         user_id = 1 if len(curr_ids) == 0 else max(curr_ids) + 1
 
         transport.user_id = user_id
@@ -269,17 +270,20 @@ class BaseLegacyActor(BaseActor):
                 command_id=command_id,
                 consumer_id=self.name,
                 actor=self,
-                loop=self.loop,
                 transport=transport,
             )
-        except clu.CommandError as ee:
+        except clu.CommandError:
             self.write(
                 "f",
-                {"text": f"Could not parse the command string: {ee!r}"},
-                user_id=user_id,
+                {"error": f"Could not parse command {command_str_s!r}"},
             )
             return
 
+        self.log.info(
+            f"New command received: {command_body   !r} "
+            f"(commander_id={commander_id!r}, command_id={command_id!r})"
+        )
+
         return self.parse_command(command)
 
     @staticmethod
@@ -436,6 +440,7 @@ class BaseLegacyActor(BaseActor):
         concatenate: bool = True,
         broadcast: bool = False,
         validate: bool = True,
+        write_to_log: bool = True,
         **kwargs,
     ):
         """Writes a message to user(s).
@@ -465,6 +470,10 @@ class BaseLegacyActor(BaseActor):
         validate
             Validate the reply against the actor model. This is ignored if the actor
             was not started with knowledge of its own schema.
+        write_to_log
+            Whether to write the reply to the log. Defaults to yes but
+            it may be useful to prevent large repetitive replies cluttering
+            the log.
         kwargs
             Keyword arguments that will be added to the message. If a keyword
             is both in ``message`` and in ``kwargs``, the value in ``kwargs``
@@ -491,10 +500,33 @@ class BaseLegacyActor(BaseActor):
                 user_id=user_id,
                 command_id=command_id,
                 concatenate=concatenate,
+                write_to_log=write_to_log,
             )
 
-    def _write_internal(self, reply: Reply, user_id=0, command_id=0, concatenate=True):
-        """Writes reply to users."""
+    def _write_internal(
+        self,
+        reply: Reply,
+        user_id=0,
+        command_id=0,
+        concatenate=True,
+        write_to_log: bool = True,
+    ):
+        """Writes reply to users.
+
+        Parameters
+        ----------
+        reply
+            The reply object to output to users.
+        user_id
+            The user to which we are replying.
+        command_id
+            The command emitting this message.
+        write_to_log
+            Whether to write the reply to the log. Defaults to yes but
+            it may be useful to prevent large repetitive replies cluttering
+            the log.
+
+        """
 
         command = cast(Command, reply.command)
         message = reply.message
@@ -546,7 +578,7 @@ class BaseLegacyActor(BaseActor):
                 if global_transport is not None and global_transport == transport:
                     transport.write(msg)
 
-            if self.log:
+            if self.log and write_to_log:
                 log_reply(self.log, reply.message_code, full_msg_str)
 
 
