@@ -20,6 +20,7 @@ from typing import Any, List, TypeVar
 
 import click
 from click.decorators import group, pass_obj
+from click_aliases import ClickAliasedGroup
 from unclick.core import command_to_json
 
 from sdsstools.logger import SDSSLogger
@@ -279,7 +280,7 @@ class CluCommand(click.Command):
             return True
 
 
-class CluGroup(click.Group):
+class CluGroup(ClickAliasedGroup):
     """Override :py:class:`click.Group`.
 
     Makes all child commands instances of `.CluCommand`.
@@ -295,7 +296,7 @@ class CluGroup(click.Group):
             kwargs["cls"] = CluCommand
 
         def decorator(f):
-            cmd = click.decorators.command(*args, **kwargs)(f)
+            cmd = ClickAliasedGroup.command(self, *args, **kwargs)(f)
             self.add_command(cmd)
             return cmd
 
@@ -478,10 +479,10 @@ def help_(ctx, *args, parser_command):
         message.append(line)
 
     if isinstance(command.actor, (actor.AMQPActor, actor.JSONActor)):
-        return command.finish(help=message)
+        return command.finish(help=message, write_to_log=False)
     else:
         for line in message:
-            command.warning(help=line)
+            command.warning(help=line, write_to_log=False)
         return command.finish()
 
 
@@ -510,9 +511,9 @@ def get_command_model(
     model_dict = json.loads(model_str)
 
     if isinstance(command.actor, LegacyActor):
-        return command.finish(command_model=model_str)
+        return command.finish(command_model=model_str, write_to_log=False)
 
-    return command.finish(command_model=model_dict)
+    return command.finish(command_model=model_dict, write_to_log=False)
 
 
 @command_parser.command(name="keyword")
@@ -532,9 +533,12 @@ def keyword(command, *args, keyword):
     lines = json.dumps(schema, indent=2).splitlines()[1:]
     max_length = max([len(line) for line in lines])
 
-    command.info(text=f"{keyword} = {{".ljust(max_length, " "))
+    command.info(text=f"{keyword} = {{".ljust(max_length, " "), write_to_log=False)
     for line in lines:
-        command.info(text=line.replace('"', "").ljust(max_length, " "))
+        command.info(
+            text=line.replace('"', "").ljust(max_length, " "),
+            write_to_log=False,
+        )
 
     command.finish()
 
