@@ -18,6 +18,7 @@ import logging
 import re
 
 from typing import (
+    TYPE_CHECKING,
     Any,
     Callable,
     Coroutine,
@@ -31,6 +32,10 @@ from typing import (
 )
 
 
+if TYPE_CHECKING:
+    from clu.base import MessageCode
+
+
 __all__ = [
     "CommandStatus",
     "StatusMixIn",
@@ -75,7 +80,6 @@ COMMAND_STATUS_TO_CODE: Dict[str, str] = {
 
 
 class CommandStatus(Maskbit):
-
     DONE = enum.auto()
     CANCELLED = enum.auto()
     FAILED = enum.auto()
@@ -93,7 +97,6 @@ class CommandStatus(Maskbit):
     ALL_STATES = READY | ACTIVE_STATES | DONE_STATES
 
     def __init__(self, *args):
-
         self.code: str | None
         if self.name and self.name.upper() in COMMAND_STATUS_TO_CODE:
             self.code = COMMAND_STATUS_TO_CODE[self.name.upper()]
@@ -104,6 +107,8 @@ class CommandStatus(Maskbit):
     def is_combination(self) -> bool:
         """Returns True if a flag is a combination."""
 
+        assert isinstance(self.value, int)
+
         if bin(self.value).count("1") > 1:
             return True
         return False
@@ -191,7 +196,6 @@ class StatusMixIn(Generic[MaskbitType]):
         callback_func: Optional[Callable[[MaskbitType], Any]] = None,
         call_now: bool = False,
     ):
-
         self.flags = maskbit_flags
         self.callbacks: List[Callable[[MaskbitType], Any]] = []
         self._status: MaskbitType | None = initial_status
@@ -214,7 +218,8 @@ class StatusMixIn(Generic[MaskbitType]):
         loop = asyncio.get_event_loop()
 
         for func in self.callbacks:
-            loop.call_soon(func, self.status)
+            if self.status:
+                loop.call_soon(func, self.status)
 
     @property
     def status(self):
@@ -262,7 +267,6 @@ class CallbackMixIn(object):
         callbacks: List[Callable[[Any], Any]] = [],
         loop: Optional[asyncio.AbstractEventLoop] = None,
     ):
-
         self._callbacks = []
         for cb in callbacks:
             self.register_callback(cb)
@@ -300,7 +304,7 @@ class CallbackMixIn(object):
         """Calls the callback functions with some arguments.
 
         Coroutine callbacks are scheduled as a task. Synchronous callbacks
-        are scheduled with ``call_soon``.
+        are called immediately.
 
         """
 
@@ -315,10 +319,7 @@ class CallbackMixIn(object):
                 # Auto-dispose of the task once it completes
                 task.add_done_callback(self._running.remove)
             else:
-                # Check that the loop is running. There is a problem in which
-                # self.loop may be set before there is a running loop so we
-                # replace it with a properly running loop.
-                task = asyncio.get_event_loop().call_soon(cb, *args[:n_args])
+                cb(*args[:n_args])
 
 
 def dict_depth(d: dict) -> int:
@@ -376,7 +377,6 @@ class CaseInsensitiveDict(Dict[str, T]):
     """A dictionary that performs case-insensitive operations."""
 
     def __init__(self, values: Any):
-
         self._lc = []
 
         dict.__init__(self, values)
@@ -400,7 +400,6 @@ class CaseInsensitiveDict(Dict[str, T]):
         return dict.__getitem__(self, self.__get_key__(key))
 
     def __setitem__(self, key, value):
-
         if key.lower() not in self._lc:
             self._lc.append(key.lower())
             dict.__setitem__(self, key, value)
@@ -417,10 +416,8 @@ class CaseInsensitiveDict(Dict[str, T]):
 def cli_coro(f):
     """Decorator function that allows defining coroutines with click."""
 
-    f = asyncio.coroutine(f)
-
     def wrapper(*args, **kwargs):
-        loop = asyncio.get_event_loop()
+        loop = asyncio.new_event_loop()
         return loop.run_until_complete(f(*args, **kwargs))
 
     return functools.update_wrapper(wrapper, f)
@@ -474,7 +471,6 @@ async def as_complete_failer(
             break
 
     if failed:
-
         # Cancel tasks
         [task.cancel() for task in tasks]
 
@@ -494,7 +490,7 @@ async def as_complete_failer(
 
 def log_reply(
     log: logging.Logger,
-    message_code: str,
+    message_code: MessageCode,
     message: str,
     use_message_code: bool = False,
 ):
@@ -510,7 +506,7 @@ def log_reply(
     }
 
     if use_message_code:
-        log.log(code_dict[message_code], message)
+        log.log(code_dict[message_code.value], message)
     else:
         # Sets the REPLY log level
         log_level_no = REPLY
@@ -544,6 +540,7 @@ class ActorHandler(logging.Handler):
     filter_warnings
         A list of warning classes that will be issued to the actor. Subclasses
         of the filter warning are accepted, any other warnings will be ignored.
+
     """
 
     def __init__(
@@ -554,7 +551,6 @@ class ActorHandler(logging.Handler):
         code_mapping: Optional[Dict[int, str]] = None,
         filter_warnings: Optional[List[Type[Warning]]] = None,
     ):
-
         self.actor = actor
         self.keyword = keyword
 
@@ -562,7 +558,7 @@ class ActorHandler(logging.Handler):
             logging.DEBUG: "d",
             logging.INFO: "i",
             logging.WARNING: "w",
-            logging.ERROR: "f",
+            logging.ERROR: "e",
         }
 
         if code_mapping:
@@ -602,7 +598,6 @@ class ActorHandler(logging.Handler):
                 asyncio.create_task(result)
 
     def _filter_warning(self, warning_category_groups):
-
         warning_category, warning_text = warning_category_groups.groups()
         message_lines = [f"{warning_text} ({warning_category})"]
 
